@@ -7,7 +7,7 @@ from alpa.cli.local_repo import (
     commit,
     pull,
     push,
-    list,
+    list_,
     genspec,
     add,
     get_pkg_archive,
@@ -29,9 +29,10 @@ def entry_point() -> None:
 
 @entry_point.command("clone")
 @click.argument("url", type=str)
-def clone(url: str) -> None:
+@click.option("--fork", is_flag=True, default=False)
+def clone(url: str, fork: bool) -> None:
     """Clone and prepare Alpa repository"""
-    AlpaRepo.clone(url)
+    AlpaRepo.clone(url, fork)
 
 
 entry_point.add_command(create)
@@ -43,7 +44,7 @@ entry_point.add_command(switch)
 entry_point.add_command(commit)
 entry_point.add_command(pull)
 entry_point.add_command(push)
-entry_point.add_command(list)
+entry_point.add_command(list_)
 entry_point.add_command(genspec)
 entry_point.add_command(add)
 entry_point.add_command(get_pkg_archive)
