@@ -8,8 +8,9 @@ from typing import Optional
 
 import click
 from click import UsageError
-from github import Github, Issue, PullRequest
+from github import Github, Issue, PullRequest, UnknownObjectException
 
+from alpa.config.local_config import Config
 from alpa.constants import GH_API_TOKEN_NAME, GH_WRITE_ACCESS
 from alpa.messages import NO_GH_API_KEY_FOUND, RETURNING_CLONE_URL_MSG
 
@@ -42,9 +43,14 @@ class GithubRepo:
 
         return upstream.clone_url
 
+    @property
     def is_fork(self) -> bool:
         return self._repo.fork
 
+    @property
+    def api_user(self) -> str:
+        return self._api.get_user().login
+
     def has_write_access(self, user: str) -> bool:
         for collaborator in self._repo.get_collaborators():
             if collaborator.login != user:
@@ -57,7 +63,7 @@ class GithubRepo:
         return False
 
     def get_upstream(self) -> Optional["GithubRepo"]:
-        if not self.is_fork():
+        if not self.is_fork:
             return None
 
         namespace, repo_name = self._repo.source.full_name.split("/")
@@ -80,22 +86,35 @@ class GithubRepo:
             title, body, base=target_branch, head=source_branch
         )
 
+    def pr_exists(self, id: int) -> bool:
+        try:
+            self._repo.get_pull(id)
+            return True
+        except UnknownObjectException:
+            return False
+
 
 class GithubAPI:
-    def __init__(self) -> None:
-        self._gh_api = Github(self._get_access_token())
+    def __init__(self, repo_name: str) -> None:
+        self._gh_api = Github(self._get_access_token(repo_name))
 
     @property
     def gh_user(self) -> str:
         return self._gh_api.get_user().login
 
     @staticmethod
-    def _get_access_token() -> str:
-        access_token = getenv(GH_API_TOKEN_NAME)
-        if access_token is None:
+    def _get_access_token(repo_name: str) -> str:
+        access_token_env = getenv(GH_API_TOKEN_NAME)
+        if access_token_env is not None:
+            return access_token_env
+
+        # accessing config file here since it does not make sense to do it elsewhere,
+        # but in the future the config will be useful in many places
+        access_token_config = Config.get_config(repo_name)
+        if access_token_config is None:
             raise UsageError(NO_GH_API_KEY_FOUND.format(token=GH_API_TOKEN_NAME))
 
-        return access_token
+        return access_token_config.gh_api_token
 
     def get_repo(self, namespace: str, repo_name: str) -> GithubRepo:
         return GithubRepo(self._gh_api, namespace, repo_name)
